from xen.xend.server.DevController import DevController
from xen.xend.XendLogging import log
from xen.xend import XendRoot

from xen.xend.XendError import VmError
from xen.xend import sxp
import xen.xend
import os, os.path

def spawn_detached(path, args, env):
    log.debug("Spawn: " + str(args))
    p = os.fork()
    if p == 0:
        os.spawnve(os.P_NOWAIT, path, args, env)
        os._exit(0)
    else:
        os.waitpid(p, 0)
        
CONFIG_ENTRIES = ['type', 'vncdisplay', 'vnclisten', 'vncpasswd', 'vncunused',
                  'display', 'xauthority', 'keymap' ]

class VfbifController(DevController):
    """Virtual frame buffer controller. Handles all vfb devices for a domain.
    Note that we only support a single vfb per domain at the moment.
    """

    def __init__(self, vm):
        DevController.__init__(self, vm)
        
    def getDeviceDetails(self, config):
        """@see DevController.getDeviceDetails"""

        back = dict([(k, sxp.child_value(config, k)) for k in CONFIG_ENTRIES
                     if sxp.child_value(config, k) is not None])

        return (0, back, {})

    def configuration(self, devid):
        r = DevController.configuration(self, devid)

        devinfo = self.readBackend(devid, *CONFIG_ENTRIES)
        for i in range(len(CONFIG_ENTRIES)):
            if devinfo[i] is not None:
                r.append([CONFIG_ENTRIES[i], devinfo[i]])
        return r
                        

    def createDevice(self, config):
        DevController.createDevice(self, config)
        # old frontend compatibility
        self.vm.writeDom("console/use_graphics", "1")
        # /old
        args = [ xen.util.auxbin.pathTo("qemu-dm"),
                 "-M", "xenpv",
                 "-d", "%d" % self.vm.getDomid(),
                 "-domain-name", self.vm.getName() ]
        t = sxp.child_value(config, "type")
        if t == "vnc":
            passwd = None
            if sxp.child_value(config, "vncpasswd") is not None:
                passwd = sxp.child_value(config, "vncpasswd")
            else:
                passwd = xen.xend.XendRoot.instance().get_vncpasswd_default()
            vncopts = ""
            if passwd:
                self.vm.storeVm("vncpasswd", passwd)
                vncopts = vncopts + ",password"
                log.debug("Stored a VNC password for vfb access")
            else:
                log.debug("No VNC passwd configured for vfb access")

            if XendRoot.instance().get_vnc_tls():
                vncx509certdir = XendRoot.instance().get_vnc_x509_cert_dir()
                vncx509verify = XendRoot.instance().get_vnc_x509_verify()

                if not os.path.exists(vncx509certdir):
                    raise "VNC x509 certificate dir does not exist"

                if vncx509verify:
                    vncopts = vncopts + ",tls,x509verify=%s" % vncx509certdir
                else:
                    vncopts = vncopts + ",tls,x509=%s" % vncx509certdir

            vnclisten = sxp.child_value(config, 'vnclisten',
                                        xen.xend.XendRoot.instance().get_vnclisten_address())
            vncdisplay = sxp.child_value(config, 'vncdisplay', 0)
            args += ['-vnc', "%s:%d%s" % (vnclisten, int(vncdisplay), vncopts)]

            if sxp.child_value(config, 'vncunused', 0):
                args += ['-vncunused']

            if sxp.child_value(config, "keymap") is not None:
                args += ["-k", "%s" % sxp.child_value(config, "keymap")]
            else:
                if XendRoot.instance().get_keymap():
                    args += ["-k", "%s" % XendRoot.instance().get_keymap()]

            spawn_detached(args[0], args, os.environ)
        elif t == "sdl":
            env = dict(os.environ)
            if sxp.child_value(config, "display") is not None:
                env['DISPLAY'] = sxp.child_value(config, "display")
            if sxp.child_value(config, "xauthority") is not None:
                env['XAUTHORITY'] = sxp.child_value(config, "xauthority")
            spawn_detached(args[0], args, env)
        else:
            raise VmError('Unknown vfb type %s (%s)' % (t, repr(config)))

class VkbdifController(DevController):
    """Virtual keyboard controller. Handles all vkbd devices for a domain.
    """

    def getDeviceDetails(self, config):
        """@see DevController.getDeviceDetails"""
        devid = 0
        back = {}
        front = {}
        return (devid, back, front)
